///File : PanelStats.cpp
//------------------------------------------------------------------------------
/*
    This file is part of forbidden2forget.

    Forbidden2forget is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Forbidden2forget is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with forbidden2forget.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PanelStats.hpp"

//------------------------------------------------------------------------------
BEGIN_EVENT_TABLE(PanelStats, wxPanel)
    EVT_TIMER(Id_Timer, PanelStats::OnTimer)
END_EVENT_TABLE()
//------------------------------------------------------------------------------
PanelStats::PanelStats( wxWindow* parent, wxWindowID id, MainFrame *PtToMainFrame, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
{
    m_PtToMainFrame = PtToMainFrame;
    m_F2fVerbs = m_PtToMainFrame->GetF2fVerbs();

	wxBoxSizer* MainSizer = new wxBoxSizer( wxVERTICAL );

	wxStaticText* LabelTitleGeneralInfo = new wxStaticText(this, wxID_ANY, _("General information :"), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE);
	LabelTitleGeneralInfo->SetFont(wxFont(15, wxFONTFAMILY_DECORATIVE, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD, true));
	MainSizer->Add(LabelTitleGeneralInfo, 0, wxEXPAND | wxALL, 5);

	m_LabelInfo = new wxStaticText(this, wxID_ANY, _T(""));
	MainSizer->Add(m_LabelInfo, 0, wxEXPAND | wxALL, 5);

	wxStaticText* LabelTitleStats = new wxStaticText(this, wxID_ANY, _("Statistic :"), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE);
	LabelTitleStats->SetFont(wxFont(15, wxFONTFAMILY_DECORATIVE, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD, true));
	MainSizer->Add(LabelTitleStats, 0, wxEXPAND | wxALL, 5);

	m_LabelStats = new wxStaticText(this, wxID_ANY, _T(""));
	MainSizer->Add(m_LabelStats, 0, wxEXPAND | wxALL, 5);
	m_GaugePerCentFinish = new wxGauge(this, wxID_ANY, 100);
	MainSizer->Add(m_GaugePerCentFinish, 0, wxEXPAND | wxALL, 5);

	SetSizer( MainSizer );

	wxTimer *myTimer = new wxTimer(this, Id_Timer);
	myTimer->Start(1000);

	Refresh();
}
//------------------------------------------------------------------------------

void PanelStats::Refresh()
{
    if(m_F2fVerbs->IsOk() && !m_F2fVerbs->IsProtected())
    {
        int value = 0;

        if(((m_F2fVerbs->GetTotalVerbsAtBegining() * NB_LEFT_PER_VERB) - m_F2fVerbs->GetTotalLeft()) != 0)
        {
            value = int(double(m_F2fVerbs->GetTotalVerbsAtBegining() * NB_LEFT_PER_VERB - m_F2fVerbs->GetTotalLeft()) / double(m_F2fVerbs->GetTotalVerbsAtBegining() * NB_LEFT_PER_VERB) * double(100));
        }

        wxString labelContent;
        labelContent << _("Different verbs at begining : ") << m_F2fVerbs->GetTotalVerbsAtBegining() << _T(".\n");
        labelContent << _("Verbs at begining : ") << m_F2fVerbs->GetTotalVerbsAtBegining() * NB_LEFT_PER_VERB << _T(".\n");
        labelContent << _("Current number of different verbs : ") << m_F2fVerbs->GetTotalLeft() << _T(".\n");
        labelContent << _("Current number of verbs : ") << m_F2fVerbs->GetTotalVerbs() << _T(".\n");
        labelContent << _("Total errors number : ") << m_F2fVerbs->GetErrorNumber() << _T(".\n");
        labelContent << value << _("% finish.");

        m_LabelStats->SetLabel(labelContent);
        m_GaugePerCentFinish->SetValue(value);

        labelContent = _T("");
        labelContent << _("Your name : ") << wxString(m_F2fVerbs->GetPupileName().c_str(), wxConvUTF8) << _T(".\n");
        labelContent << _("Your level : ") << wxString(m_F2fVerbs->GetLevel().c_str(), wxConvUTF8) << _T(".\n");
        if(m_F2fVerbs->IsPassword())
        {
            labelContent << _("You are using a password.\n");
        }
        else
        {
            labelContent << _("You are not using a password.\n");
        }

        if(m_F2fVerbs->IsFinish())
        {
            labelContent << _("Time you spend during this session : ") << FormatTimeSpan(wxTimeSpan()) << _T(".\n");
            labelContent << _("Total time you spend : ") << FormatTimeSpan(wxTimeSpan(0,0, m_F2fVerbs->GetWorkDuration())) << _T(".");
        }
        else
        {
            labelContent << _("Time you spend during this session : ") << FormatTimeSpan(wxDateTime::Now() - m_PtToMainFrame->GetBiginWorkDateTime()) << _T(".\n");
            labelContent << _("Total time you spend : ") << FormatTimeSpan((wxDateTime::Now() - m_PtToMainFrame->GetBiginWorkDateTime()) + wxTimeSpan(0,0,m_F2fVerbs->GetWorkDuration())) << _T(".");
        }

        m_LabelInfo->SetLabel(labelContent);
    }
    else
    {
        wxString labelContent;
        labelContent << _("Different verbs at begining : ") << 0 << _T(".\n");
        labelContent << _("Verbs at begining : ") << 0 << _T(".\n");
        labelContent << _("Current number of different verbs : ") << 0 << _T(".\n");
        labelContent << _("Current number of verbs : ") << 0 << _T(".\n");
        labelContent << _("Total errors number : ") << 0 << _T(".\n");
        labelContent << 0 << _("% finish.");

        m_LabelStats->SetLabel(labelContent);
        m_GaugePerCentFinish->SetValue(0);

        m_LabelInfo->SetLabel(_T("\n\n\n\n\n"));
    }
}
//------------------------------------------------------------------------------

wxString PanelStats::FormatTimeSpan(wxTimeSpan timeSpan)
{
    wxString str;
    str << timeSpan.GetMinutes() << _(" minutes");
    return str;
}
//------------------------------------------------------------------------------

void PanelStats::OnTimer(wxTimerEvent &event)
{
    Refresh();
}
