///File : PanelVerbs.cpp
//------------------------------------------------------------------------------
/*
    This file is part of forbidden2forget.

    Forbidden2forget is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Forbidden2forget is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with forbidden2forget.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PanelVerbs.hpp"
#include "findFile.hpp"
//------------------------------------------------------------------------------
BEGIN_EVENT_TABLE(PanelVerbs, wxPanel)
    // Bt Valider
    EVT_BUTTON(wxID_OK,  PanelVerbs::OnValider)

    //Text box
    EVT_TEXT_ENTER(Id_TxtBoxValid, PanelVerbs::OnValider)
END_EVENT_TABLE()

//------------------------------------------------------------------------------
void PanelVerbs::OnNew(int lvl)
{
    if(lvl >= 2 && lvl <= 5)
    {
        wxString originFilename;
        originFilename << _T("data/") << lvl << _T(".f2f");
        originFilename = findFile(originFilename);

        wxString selectedFile = wxFileSelector(_("Save..."), _T(""), _T(""), _("Forbidden to Forget files (*.f2f)|*.f2f"), _("Forbidden to Forget files (*.f2f)|*.f2f|All files|*"), wxFD_SAVE);
        if(selectedFile != _T(""))
        {

            wxString userName = wxGetTextFromUser(_("Enter your name and surname (ex: GILLE Colin)"), _("User name"), _T(""), this);

            //Tjr extension
            if(selectedFile.Mid(selectedFile.Len() - 4, selectedFile.Len()) != _T(".f2f"))
            {
                selectedFile << _T(".f2f");
            }

            if(userName != _T(""))
            {
                // ON sauvegarde
                if(m_F2fVerbs->IsOk() && !m_F2fVerbs->IsProtected())
                {
                    OnSave();
                }

                try
                {
                    m_F2fVerbs->New(std::string(originFilename.mb_str(wxConvUTF8)), std::string(selectedFile.mb_str(wxConvUTF8)), std::string(userName.mb_str(wxConvUTF8)));
                }
                catch(const std::exception& e) //Rattrape toutes les exceptions
                {
                    wxMessageBox(_("Error while loading file : ") + wxString(e.what(), wxConvUTF8), _("Error..."),  wxOK | wxICON_ERROR);
                }

                m_PtToMainFrame->ResetBiginWorkDateTime();

                Valider();
            }
        }
    }
}

//------------------------------------------------------------------------------
void PanelVerbs::OnOpen(wxString selectedFile)
{
    if(selectedFile == _T(""))
    {
         selectedFile = wxFileSelector(_("Select your file"), _T(""), _T(""), _("Forbidden to Forget files (*.f2f)|*.f2f"), _("Forbidden to Forget files (*.f2f)|*.f2f|All files|*"));
    }


    if(selectedFile != _T(""))
    {
        if(wxFileExists(selectedFile))
        {
            //On sauvegarde...
            if(m_F2fVerbs->IsOk() && !m_F2fVerbs->IsProtected())
            {
                OnSave();
            }

            try
            {
                m_F2fVerbs->LoadFromFile(std::string(selectedFile.mb_str(wxConvUTF8)));
            }
            catch(const std::exception& e) //Rattrape toutes les exceptions
            {
                wxMessageBox(_("Error while loading file : ") + wxString(e.what(), wxConvUTF8), _("Error..."),  wxOK | wxICON_ERROR);
            }
        }
        else
        {
            wxMessageBox(_("File doesn't exists."), _("Open"), wxOK | wxICON_ERROR);
        }
    }

    if(m_F2fVerbs->IsOk())
    {
        while(m_F2fVerbs->IsProtected())
        {
            m_F2fVerbs->TryPassword(std::string(wxGetPasswordFromUser(_("Enter the password to open this file :"), _("Prtected file"), _T(""), this).mb_str(wxConvUTF8)));
            if(m_F2fVerbs->IsProtected() && wxMessageBox(_("Bad password. Do you want to retry ?"), _("Bad password..."),  wxYES_NO | wxICON_ERROR) == wxNO)
            {
                break;
            }
        }

        if(m_F2fVerbs->IsProtected())
        {
            OnClose();
        }

        m_PtToMainFrame->ResetBiginWorkDateTime();
    }
    else
    {
        OnClose();
    }

    Valider();
}
//------------------------------------------------------------------------------

void PanelVerbs::OnSave()
{
    if(!m_F2fVerbs->IsFinish())
    {
        wxLongLong myLongLongElapsedSeconds = (wxDateTime::Now() - m_PtToMainFrame->GetBiginWorkDateTime()).GetSeconds();
        m_F2fVerbs->AdToElapsedSeconds(int(myLongLongElapsedSeconds.ToLong()));
        m_PtToMainFrame->ResetBiginWorkDateTime();
    }

    try
    {
        m_F2fVerbs->Save();
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error while saving : ") + wxString(e.what(), wxConvUTF8), _("Error..."),  wxOK | wxICON_ERROR);
    }

    Refresh();
}
//------------------------------------------------------------------------------

void PanelVerbs::OnFrameClose()
{
    if(m_F2fVerbs->IsOk() && !m_F2fVerbs->IsProtected())
    {
        OnSave();
        wxMessageBox(_("Your file has been saved."),_("Close"), wxICON_INFORMATION | wxOK, this);
    }
}
//------------------------------------------------------------------------------*/

void PanelVerbs::OnClose()
{
    if(m_F2fVerbs->IsOk() && !m_F2fVerbs->IsProtected())
    {
        OnSave();
        wxMessageBox(_("Your file has been saved."),_("Close"), wxICON_INFORMATION | wxOK, this);
    }

    m_F2fVerbs->Clear();
    m_TxtCtrlBV->SetValue(_T(""));
    m_TxtCtrlPret->SetValue(_T(""));
    m_TxtCtrlPP->SetValue(_T(""));
    Refresh();
}
//------------------------------------------------------------------------------

void PanelVerbs::OnChgPassword()
{
    int msgBoxResult = wxMessageBox(_("Do you want tu use a password ?"),_("Password"), wxYES_NO | wxCANCEL | wxYES_DEFAULT | wxICON_QUESTION , this);
    if(msgBoxResult == wxYES)
    {
        wxString newPassword = wxGetPasswordFromUser(_("Enter the new password :"), _("Change password"), _T(""), this);
        if(newPassword == wxGetPasswordFromUser(_("Re-enter the new password :"), _("Change password"), _T(""), this))
        {
            m_F2fVerbs->SetPassword(std::string(newPassword.mb_str(wxConvUTF8)));
        }
        else
        {
            wxMessageBox(_("The passwords are different, no change processed."), _("Password"), wxOK | wxICON_EXCLAMATION);
        }
    }
    else if(msgBoxResult == wxNO)
    {
        wxMessageBox(_("No password used."), _("Password"), wxOK | wxICON_INFORMATION);
        m_F2fVerbs->SetPassword("");
    }
}
//------------------------------------------------------------------------------
void PanelVerbs::OnValider(wxCommandEvent& WXUNUSED(event))
{
    Valider();
}
//------------------------------------------------------------------------------
void PanelVerbs::Valider()
{
    if(m_F2fVerbs->IsOk())
    {
        if(m_F2fVerbs->IsCorrecting())
        {
            m_TxtCtrlBV->SetValue(_T(""));
            m_TxtCtrlPret->SetValue(_T(""));
            m_TxtCtrlPP->SetValue(_T(""));

            m_TxtCtrlBV->SetFocus();

            m_F2fVerbs->GetNewVerb();
        }
        else
        {
            // Par defaut, tout est vert (pr simplifier).
            m_TxtCtrlBV->SetBackgroundColour(*wxGREEN);
            m_TxtCtrlPP->SetBackgroundColour(*wxGREEN);
            m_TxtCtrlPret->SetBackgroundColour(*wxGREEN);

            if(!m_F2fVerbs->Correct(std::string(m_TxtCtrlBV->GetValue().mb_str(wxConvUTF8)),std::string(m_TxtCtrlPret->GetValue().mb_str(wxConvUTF8)), std::string(m_TxtCtrlPP->GetValue().mb_str(wxConvUTF8))))
            {
                wxString RealBV = wxString(m_F2fVerbs->GetCurrentBV().c_str(), wxConvUTF8);
                wxString RealPret  = wxString(m_F2fVerbs->GetCurrentPret().c_str(), wxConvUTF8);
                wxString RealPP  = wxString(m_F2fVerbs->GetCurrentPP().c_str(), wxConvUTF8);

                if(RealBV != m_TxtCtrlBV->GetValue())
                {
                    m_TxtCtrlBV->SetValue(RealBV);
                    m_TxtCtrlBV->SetBackgroundColour(*wxRED);
                }

                if(RealPret != m_TxtCtrlPret->GetValue())
                {
                    m_TxtCtrlPret->SetValue(RealPret);
                    m_TxtCtrlPret->SetBackgroundColour(*wxRED);
                }

                if(RealPP != m_TxtCtrlPP->GetValue())
                {
                    m_TxtCtrlPP->SetValue(RealPP);
                    m_TxtCtrlPP->SetBackgroundColour(*wxRED);
                }
            }
        }
    }

    if(m_F2fVerbs->IsFinish())
    {
        wxLongLong myLongLongElapsedSeconds = (wxDateTime::Now() - m_PtToMainFrame->GetBiginWorkDateTime()).GetSeconds();
        m_F2fVerbs->AdToElapsedSeconds(int(myLongLongElapsedSeconds.ToLong()));
        m_PtToMainFrame->ResetBiginWorkDateTime();
    }

    Refresh();
}

//------------------------------------------------------------------------------
void PanelVerbs::Refresh()
{
    if(m_F2fVerbs->IsOk() && !m_F2fVerbs->IsProtected())
    {
        m_TxtCtrlBV->Enable(true);
        m_TxtCtrlPret->Enable(true);
        m_TxtCtrlPP->Enable(true);

        m_BtValider->Enable(true);

        m_PtToMainFrame->EnableMenus(true);

        wxString NewTitle;
        NewTitle << _("Forbidden 2 Forget") << _T(" - ") << wxString(m_F2fVerbs->GetPupileName().c_str(), wxConvUTF8);
        m_PtToMainFrame->SetTitle(NewTitle);

        if(!m_F2fVerbs->IsFinish())
        {
            wxString labelContent;
            labelContent << _("Verb: ") << wxString(m_F2fVerbs->GetCurrentFr().c_str(), wxConvUTF8) << _T(" (") << m_F2fVerbs->GetCurrentNbLeft() << _T("x)");
            m_LabelVerb->SetLabel(labelContent);
        }
        else
        {
            m_LabelVerb->SetLabel(_("Finish !!"));

            m_TxtCtrlBV->SetBackgroundColour(*wxWHITE);
            m_TxtCtrlPP->SetBackgroundColour(*wxWHITE);
            m_TxtCtrlPret->SetBackgroundColour(*wxWHITE);

            m_TxtCtrlBV->Enable(false);
            m_TxtCtrlPret->Enable(false);
            m_TxtCtrlPP->Enable(false);
            m_BtValider->Enable(false);
            m_TxtCtrlBV->SetValue(_T(""));
            m_TxtCtrlPret->SetValue(_T(""));
            m_TxtCtrlPP->SetValue(_T(""));
        }
    }
    else
    {
        m_TxtCtrlBV->Enable(false);
        m_TxtCtrlPret->Enable(false);
        m_TxtCtrlPP->Enable(false);

        m_BtValider->Enable(false);

        m_PtToMainFrame->EnableMenus(false);

        m_PtToMainFrame->SetTitle(_("Forbidden 2 Forget"));

        m_LabelVerb->SetLabel(_T(""));

        m_TxtCtrlBV->SetBackgroundColour(*wxWHITE);
        m_TxtCtrlPP->SetBackgroundColour(*wxWHITE);
        m_TxtCtrlPret->SetBackgroundColour(*wxWHITE);
    }

    if(m_F2fVerbs->IsCorrecting())
    {
        //On ne peut pas modifier une correcion.
        m_TxtCtrlBV->SetEditable(false);
        m_TxtCtrlPret->SetEditable(false);
        m_TxtCtrlPP->SetEditable(false);
    }
    else
    {
        m_TxtCtrlBV->SetEditable(true);
        m_TxtCtrlPret->SetEditable(true);
        m_TxtCtrlPP->SetEditable(true);

        m_TxtCtrlBV->SetBackgroundColour(*wxWHITE);
        m_TxtCtrlPP->SetBackgroundColour(*wxWHITE);
        m_TxtCtrlPret->SetBackgroundColour(*wxWHITE);
    }
    m_TxtCtrlBV->Refresh();
    m_TxtCtrlPP->Refresh();
    m_TxtCtrlPret->Refresh();

    m_PtToMainFrame->RefreshPanelStats();
}

//------------------------------------------------------------------------------
PanelVerbs::PanelVerbs( wxWindow* parent, wxWindowID id, MainFrame *PtToMainFrame, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
{
    m_PtToMainFrame = PtToMainFrame;
    m_F2fVerbs = PtToMainFrame->GetF2fVerbs();

    wxBoxSizer *MainSizer = new wxBoxSizer( wxHORIZONTAL );


    wxBoxSizer *RightSizer = new wxBoxSizer( wxVERTICAL );
        //Verb à faire
    m_LabelVerb = new wxStaticText(this, wxID_ANY,  _T(""));
    RightSizer->Add(m_LabelVerb, 0, wxALL, 5);

        //Linge pr deco
    wxStaticLine *decoLine1 = new wxStaticLine(this);
    RightSizer->Add(decoLine1, 0, wxEXPAND | wxTOP | wxBOTTOM | wxRIGHT, 5 );

        //Txt BV
    wxStaticText *LabelBV = new wxStaticText(this, wxID_ANY, _("Base :"));
    RightSizer->Add(LabelBV, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5 );
    m_TxtCtrlBV = new wxTextCtrl(this, Id_TxtBoxValid, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER );
    RightSizer->Add(m_TxtCtrlBV, 0, wxEXPAND | wxALL, 5 );
    RightSizer->Add(0,0,1,0,0); //Espace

        //Txt Preterit
    wxStaticText *LabelPret = new wxStaticText(this, wxID_ANY, _("Preterit :"));
    RightSizer->Add(LabelPret, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5 );
    m_TxtCtrlPret = new wxTextCtrl(this, Id_TxtBoxValid, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER );
    RightSizer->Add(m_TxtCtrlPret, 0, wxEXPAND | wxALL, 5 );
    RightSizer->Add(0,0,1,0,0); //Espace

        //Txt PP
    wxStaticText *LabelPP = new wxStaticText(this, wxID_ANY, _("Past participle :"));
    RightSizer->Add(LabelPP, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5 );
    m_TxtCtrlPP = new wxTextCtrl(this, Id_TxtBoxValid, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER );
    RightSizer->Add(m_TxtCtrlPP, 0, wxEXPAND | wxALL, 5 );
    RightSizer->Add(0,0,1,0,0); //Espace

        //Linge pr deco
    wxStaticLine *decoLine2 = new wxStaticLine(this);
    RightSizer->Add(decoLine2, 0, wxEXPAND | wxTOP | wxBOTTOM | wxRIGHT, 5 );

        //Bt Valider
    m_BtValider = new wxButton(this, wxID_OK);
    RightSizer->Add(m_BtValider, 0, wxALIGN_RIGHT | wxALL, 5 );


        //Img de gauche
    wxString ImgFileName = findFile(_T("data/f2f-big.png"));
    if(wxFileExists(ImgFileName))
    {
        wxImage img(ImgFileName, wxBITMAP_TYPE_PNG);
        wxStaticBitmap *m_bitmapBigLogo = new wxStaticBitmap( this, wxID_ANY, wxBitmap(img), wxDefaultPosition, wxSize(73,368));
        MainSizer->Add( m_bitmapBigLogo, 0, wxALL | wxALIGN_CENTER, 5 );
    }

    wxStaticLine *vDecoLine3 = new wxStaticLine(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_VERTICAL );
    MainSizer->Add(vDecoLine3, 0, wxEXPAND | wxTOP | wxBOTTOM, 5 );


    MainSizer->Add(RightSizer, 1, wxEXPAND);

	SetSizer(MainSizer);

	Refresh();
}
